%AnisotropicConduction
%Calculation of conduction through structure of coupled traxels

% close all
% clear all
clc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% In this script we use (Ntraxels x 2) matrices to define inputs and
% outputs which establish the boundary conditions. The meaning of the
% various values is:
% First element: integer [1,2,3], select kind of boundary
% 1=voltage, 2=current=0, 3=connection to other traxel
% Second element: real 
% case 1=voltage value
% case 2=redundant
% case 3=other traxel number

%The parameters of the model can be changed in InitParameters()

%RunModel executes all relevant parts of the code, plots the impedance over
%the frequency and plots voltage, current and power for three different
%frequencies. The exact frequencies can be changed at the end inside the function.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%whether to parallelize computation or not
use_parallel = false;

[U,I,P,Z] = RunModel(use_parallel);

%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Initialize calculations to represent Transducers Paper structure + 3D
%additions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Ntraxels,Width,Height,Length,Rho,Sigma_y,Sigma_z,epsilon,Ctrans_y,Ctrans_z,Uin,Uou,Fmin,Fmax,Fnum]=InitParameters
    

%%%%%%%%%%%%%Specify geometric parameters
%     Ntraxels=[15,10];             %Number of traxels in the structure [y,z] 
    Ntraxels=[5,4];             %Number of traxels in the structure [y,z] 
%     Ntraxels=[9,6];             %Number of traxels in the structure [y,z] 
    
%     Width=10 / 9 * 1E-3;                %Width of traxels in m
%     Height=2e-4;               %Height of traxels in m
%     Length=20e-3;                %Length of traxels in m
    Width=8E-4;                %Width of traxels in m
    Height=1e-4;               %Height of traxels in m
    Length=5e-3;                %Length of traxels in m
    
    assignin('base','Length',Length); 
    assignin('base','Height',Height); 
    assignin('base','Width',Width);
    
    B=Width*Ntraxels;            %Breadth of structure
    Area=Width*Height;           %Crossectional area of traxel in m^2

%%%%%%%%%%%%Specify material parameters
%     Rho=0.1;               %Volume Resistivity of filament in Ohm*m
    
    Sigma_y=8e-1;                %Surface resistivity in Ohm*m^2
    Sigma_z=2e-1;                 %Surface resistivity in Ohm*m^2
    
    epsilon=8.85e-12*5; %4.5e4                 %Longitudinal Capacitance in F
    
    Ctrans_y= 8e-4;                 %Transverse (surface) Capacitance in F/m^2
    Ctrans_z= 2e-5;                 %Transverse (surface) Capacitance in F/m^2
    
%     Rho=2.8;               %Volume Resistivity of filament in Ohm*m
    Rho=2.8;               %Volume Resistivity of filament in Ohm*m
    
%     Sigma_y=2e-2;                %Surface resistivity in Ohm*m^2
% %     Sigma_z=2e-2;                %Surface resistivity in Ohm*m^2
%     Sigma_z=1.75e-2;                 %Surface resistivity in Ohm*m^2
    
    %first test
%     Sigma_y=8e-1;                %Surface resistivity in Ohm*m^2
%     Sigma_z=2e-1;                 %Surface resistivity in Ohm*m^2
    
%     Sigma_y=2e-3;                %Surface resistivity in Ohm*m^2
%     Sigma_z=2e-3;                 %Surface resistivity in Ohm*m^2
    
%     Ctrans_y= 8e-4;                 %Transverse (surface) Capacitance in F/m^2
% %     Ctrans_z= 8e-4;                 %Transverse (surface) Capacitance in F/m^2
%     Ctrans_z= 9e-4;                 %Transverse (surface) Capacitance in F/m^2
    
    %first test
%     Ctrans_y= 8e-4;                 %Transverse (surface) Capacitance in F/m^2
%     Ctrans_z= 2e-5;                 %Transverse (surface) Capacitance in F/m^2
    
%     Ctrans_y= 2.8e-4;                 %Transverse (surface) Capacitance in F/m^2
%     Ctrans_z= 2.8e-4;                 %Transverse (surface) Capacitance in F/m^2
    
    assignin('base','cutoff_x',1/(2*pi*epsilon*Rho)); 
    assignin('base','cutoff_y',1/(2*pi*Sigma_y*Ctrans_y)); 
    assignin('base','cutoff_z',1/(2*pi*Sigma_z*Ctrans_z)); 
    
    Fmin=0;                    %Lowest frequency
    Fmax=1E8;                    %Highest frequency
    Fnum=200;                   % Number of frequencies

%%%%%%%%%%%%Specify Voltages
    Uhig=1;                      %High voltage
    Ulow=0;                      %Low voltage
    
    [Uin,Uou]=AllOpenConnections(Ntraxels);
    
%%%%%%%%%%%%Specify meanders 
    [Uin,Uou]=AllMeanderInput(Ntraxels,Uin,Uou);
    
%%%%%%%%%%%%Specify terminals and ground
    Uin( 1: 1,1:2)=[1 Uhig];
%     Uin( end:end,1:2)=[1 Ulow];
    Uou( end:end,1:2)=[1 Ulow];
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function to make an interactive plot with a slider to slice through
% 3D-Data
% Input:
% N : Number of traxels in y- and z-direction [y,z]
% U : Voltage double (N(1)*N(2), inf)
% I : Current double (N(1)*N(2), inf)
% P : Power double (N(1)*N(2), inf)
% Pdens : Power density double (N(1)*N(2), inf)
% AdjustColor : Bool to dynamically adjust the colormap of the
% visualization. Useful because upper layers often have much lower values
% for e.g. voltage
% AdjustAxes : Bool to dynamically adjust the axes of the
% Graph. Turn off for showing how the values develope through the layers
% Turn on for comparing the values of one layer
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function slice_3D_plot(Length,Width,N,U,I,P,Pdens,Freq,AdjustAxes,AdjustColor)

%define generic parameters for font etc.
    FS=17;
    AFS=13;
    current_slice = 1;
    f = figure;
    
%create control objects
    ax = axes('Parent',f,'position',[0.13 0.39  0.77 0.54]);
    b = uicontrol('Parent',f,'Style','slider','Position',[81,54,23,420],...
              'value',current_slice, 'min',1, 'max',N(2),'SliderStep',[1/(N(2)-1),1/(N(2)-1)]);
          
    bgcolor = f.Color;
    bl1 = uicontrol('Parent',f,'Style','text','Position',[50,54,23,23],...
                    'String','1','BackgroundColor',bgcolor);
    bl2 = uicontrol('Parent',f,'Style','text','Position',[50,450,23,23],...
                    'String',N(2),'BackgroundColor',bgcolor);
    bl3 = uicontrol('Parent',f,'Style','text','Position',[0,240,75,23],...
                    'String','Layer '+string(round(current_slice)),'BackgroundColor',bgcolor);

%define colors for colormaps
    col=['r','g','b','y','m','c'];
    [cr,cl]=size(col);
    
%define arrays that represent axes
    x = 0:size(U,2)-1;
    %scale x-axis according to length
    x  = x / (size(U,2)-1) * Length;
    %indices i for one layer
    i = 1+(current_slice-1)*N(1):current_slice*N(1);
    MaxCol=2048;
    cm=jet(MaxCol); %Set color scales
    
    %visualization x- and y-ticks
%     if N(1) >= 15
%         num_ticks = round(N(1) / (N(1)/15));
%     else
%         num_ticks = N(1);
%     end
    ticks_x=0:Length/5:Length;
%     ticks_y=0:1:num_ticks;
    ticks_y=0:(Width*N(1)) / 5:Width*N(1);
%     px_ticks_y=linspace(0,10,numel(ticks_y));
    ticklabels_x=cellfun(@(v) sprintf('%.2g',v),num2cell(ticks_x),...
    'UniformOutput',false);
    ticklabels_y=cellfun(@(v) sprintf('%.2g',v),num2cell(ticks_y),...
    'UniformOutput',false);
    
%make visuals
    %Voltage Graph
    plots(1) = subplot(2,3,1);
    y = U(i,:);
    pU = plot(x,real(y),'linewidth',2);
    title('Voltage vs Position @'+string(Freq)+'Hz');
    xlabel("[x]=m",'fontsize',FS);
    ylabel("[U]=V",'fontsize',FS);
    set(gca,'FontSize',AFS);
    axis([-inf inf min(real(U(:))) max(real(U(:)))]);
    
	%Voltage Colormap
    RI=imref2d(size(y));
    RI.XWorldLimits=[0 10];
    RI.YWorldLimits=[0 10];
    dmin=min(real(U(:)));
    dmax=max(real(U(:)));

    plots(2) = subplot(2,3,4);
    hold on;
    grid on;
    imU = imshow(real(y),RI,[dmin dmax]);
    
    set(gca,'XTickLabels',ticklabels_x)
%     set(gca,'YTick',px_ticks_y)
    set(gca,'YTickLabels',ticklabels_y)
    colormap(gca, cm);
    colorbar;
    title('Voltage @'+string(Freq)+'Hz');
    xlabel("[x] = m",'fontsize',FS);
    ylabel("[y] = m",'fontsize',FS);
    set(gca,'FontSize',AFS);
    
    
    %Current Graph
    plots(3) = subplot(2,3,2);
    y = I(i,:);
    pI = plot(x,real(y),'linewidth',2);
    title('Current density vs Position @'+string(Freq)+'Hz');
    xlabel("[x]=m",'fontsize',FS);
    ylabel("[I]=A/m^2",'fontsize',FS);
    set(gca,'FontSize',AFS);
    axis([-inf inf min(real(I(:))) max(real(I(:)))]);
    
	%Current Colormap
    RI=imref2d(size(y));
    RI.XWorldLimits=[0 10];
    RI.YWorldLimits=[0 10];
    dmin=min(real(I(:)));
    dmax=max(real(I(:)));

    plots(4) = subplot(2,3,5);
    hold on;
    grid on;
    imI = imshow(real(y),RI,[dmin dmax]);
    set(gca,'XTickLabels',ticklabels_x)
%     set(gca,'YTick',px_ticks_y)
    set(gca,'YTickLabels',ticklabels_y)
    colormap(gca, cm);
    colorbar;
    title('Current density @'+string(Freq)+'Hz');
    xlabel("[x] = m",'fontsize',FS);
    ylabel("[y] = m",'fontsize',FS);
    set(gca,'FontSize',AFS);
    
    %Power Graph
    plots(5) = subplot(2,3,3);
    y = P(i,:);
    pP = plot(x,y,'linewidth',2);
    title('Power vs Position @'+string(Freq)+'Hz');
    xlabel("[x]=m",'fontsize',FS);
    ylabel("[P]=W",'fontsize',FS);
    set(gca,'FontSize',AFS);
    axis([-inf inf min(real(P(:))) max(real(P(:)))]);
    
	%Power Colormap
    y = Pdens(i,:);
    RI=imref2d(size(y));
    RI.XWorldLimits=[0 10];
    RI.YWorldLimits=[0 10];
    dmin=min(real(Pdens(:)));
    dmax=max(real(Pdens(:)));
    
    plots(6) = subplot(2,3,6);
    hold on;
    grid on;
    imP = imshow(real(y),RI,[dmin dmax]);
    set(gca,'XTickLabels',ticklabels_x)
%     set(gca,'YTick',px_ticks_y)
    set(gca,'YTickLabels',ticklabels_y)
    colormap(gca, hot(MaxCol));
    colorbar;
    title('Power Density @'+string(Freq)+'Hz');
    xlabel("[x] = m",'fontsize',FS);
    ylabel("[y] = m",'fontsize',FS);
    set(gca,'FontSize',AFS);
    
%set callback to slide through layers
    b.Callback = @(es,ed) slice_plot_callback(es,ed,f,bgcolor,N,U,I,P,Pdens,plots,pU,pI,pP,imU,imI,imP,AdjustColor,AdjustAxes, bl3);
    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Callback function for slice_3D_plot to update layer and slider
%Only used as callback, no direct use
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function slice_plot_callback(es,ed,f,bg,N,U,I,P,Pdens,plots,pU,pI,pP,imU,imI,imP,AdjustColor,AdjustAxes, bl3)

    %TODO: Dynamically adjust position of slider
%     pos = get(plots(4),'position');
    
    %Update slider title "Layer x"
    bl3.String = 'Layer '+string(round(es.Value));
%     bl3 = uicontrol('Parent',f,'Style','text','Position',[240,25,100,23],...
%                     'String','Layer '+string(round(es.Value)),'BackgroundColor',bg);
                
    %Set value of current slice and indices
    current_slice = round(es.Value);
    i=1+(current_slice-1)*N(1):current_slice*N(1);
    
    %get new layer Voltage
    y = real(U(i,:));
    y_cells = num2cell(y,2);

    %Update the plots
    subplot(plots(1));
    set(pU,{'YData'},y_cells);
    if AdjustAxes
        axis([-inf inf min(y(:)) max(y(:))]);
    end
    
    subplot(plots(2));
    set(imU,'CData',y);
    
    %dynamically adjust color range
    if AdjustColor
        ax = gca;
        dmin=min(y(:));
        dmax=max(y(:));
        set(ax,'CLim',[dmin,dmax]);
    end
    
    
    %get new layer Current
    y = real(I(i,:));
    y_cells = num2cell(y,2);

    %Update the plots
    subplot(plots(3));
    set(pI,{'YData'},y_cells);
    if AdjustAxes
        axis([-inf inf min(y(:)) max(y(:))]);
    end
%     axis([-inf inf 0 50]);
    
    subplot(plots(4));
    set(imI,'CData',y);
    
    %dynamically adjust color range
    if AdjustColor
        ax = gca;
        dmin=min(y(:));
        dmax=max(y(:));
        set(ax,'CLim',[dmin,dmax]);
    end
    
    
    %get new layer Power
    y = real(P(i,:));
    y_cells = num2cell(y,2);

    %Update the plots
    subplot(plots(5));
    set(pP,{'YData'},y_cells);
    if AdjustAxes
        axis([-inf inf min(y(:)) max(y(:))]);
    end
%     axis([-inf inf 0 50]);
    
    subplot(plots(6));
    y = real(Pdens(i,:));
    set(imP,'CData',y);
    
    %dynamically adjust color range
    if AdjustColor
        ax = gca;
        dmin=min(y(:));
        dmax=max(y(:));
        set(ax,'CLim',[dmin,dmax]);
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function to calculate and simulated data measured by Alexander Dijkshoorn
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [U,I,P,Z] = RunModel(use_parallel)
    

%Get parameters from designated function
    [N,Width,Height,Length,Rho,Sigma_y,Sigma_z,epsilon,Ctrans_y,Ctrans_z,Uin,Uou,Fmin,Fmax,Fnum]=InitParameters;
    
    
%Read out frequencies for sweep
    Freq = csvread('FrequencyC.txt');
    
%Calculate impedance for all frequencies
    tic
    if use_parallel
        parfor p = 1:Fnum
            [Lambda,eVec,Alpha,Z(p),Rlat(p),Gamma_y(p),Gamma_z(p)]=CalcSolution(N,Length,Width,Height,Rho,Sigma_y,Sigma_z,epsilon,Ctrans_y,Ctrans_z,Freq(p),Uin,Uou);
        end
    else
        for p = 1:Fnum
            [Lambda,eVec,Alpha,Z(p),Rlat(p),Gamma_y(p),Gamma_z(p)]=CalcSolution(N,Length,Width,Height,Rho,Sigma_y,Sigma_z,epsilon,Ctrans_y,Ctrans_z,Freq(p),Uin,Uou);
        end
    end
    toc
    
%send usefull results to workspace
    assignin('base', 'Freq', Freq);
    assignin('base', 'F_Z', [Freq; Z]);
    
    assignin('base', 'F_Gamma_y', [Freq; Gamma_y]);
    assignin('base', 'F_Gamma_z', [Freq; Gamma_z]);
    
%draw relative conductivity ellipsoid
    make_ellipsoid([Freq; Gamma_y], [Freq; Gamma_z]);
    
%make impedance graph based on previous results
    imp = Impedance_Graph(Z,Freq,0);
    
%calculate full solution for visualizations for a few specific frequencies
    Npoints = 200; %resolution for lateral direction
    for p = 2:2:8
        freq = 10^p;
        [z,U,I,P,Pdens,Z]=CalcAllProfiles(N,Length,Width,Height,Rho,Sigma_y,Sigma_z,epsilon,Ctrans_y,Ctrans_z,freq,Uin,Uou,Npoints);
        slice_3D_plot(Length,Width,N,U,I,P,Pdens,freq,1,0);
        
        %Change 3D-Visualization parameters here. Notably the
        %alphamaps and scaling
        if mod(N(1),2) == 0
            Visualize_3D(U,I,P,N, freq, 'vdown','vdown','rampup', [Length, Width, Height], false);
        else
            Visualize_3D(U,I,P,N, freq, 'vdown','rampup','rampup', [Length, Width, Height], false);
        end
        i = round(p/2)+1;
        
        %put results into a larger array for easy access
        Voltages(i,:,:) = U;
        Currents(i,:,:) = I;
        Powers(i,:,:) = P;
        Impedances(i) = Z;
    end
    U = Voltages;
    I = Currents;
    P = Powers;
    Z = Impedances;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function to have all input and output disconnected and establish base
% structure
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Uin,Uou]=AllOpenConnections(N)
    Uin(1:N(1)*N(2),1:1)=2;
    Uin(1:N(1)*N(2),2:2)=0;
    Uou(1:N(1)*N(2),1:1)=2;
    Uou(1:N(1)*N(2),2:2)=0;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function to have all input and output connected like meanders
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Uin,Uou]=AllMeanderInput(N,Uin,Uou)
    
    %Calculate evenness to differ between cases
    NHTRE = ~logical(mod(N(1),2)); %Number horizontal traxels even?
    
    %make horizontal connections for all layers
    for h=0:N(2)-1
           
        for m=2:2:N(1)-NHTRE
            Uin(m+h*N(1):m+h*N(1),    1:2)=[3 m+1+h*N(1)];
            Uin(m+1+h*N(1):m+1+h*N(1),1:2)=[3 m+h*N(1)];
        end
        
        for m=1:2:N(1)-~NHTRE
            Uou(m+h*N(1):m+h*N(1),    1:2)=[3 m+1+h*N(1)];
            Uou(m+1+h*N(1):m+1+h*N(1),1:2)=[3 m+h*N(1)];
        end
    
        
    end
    
%these commented lines create vertical meanders at the end and beginning of
%the layers so that the entire structure is effectively a single long line.
%This is not realistic as FDM doesn't do this. Still, in case you
%want this I commented it out instead of deleting.

%     %first traxel vertical connections
%     for h=1:2:N(2)-2
%             Uin(1+h*N(1):1+h*N(1),      1:2)=[3 1+(h+1)*N(1)];
%             Uin(1+(h+1)*N(1):1+(h+1)*N(1),  1:2)=[3 1+h*N(1)];
%     end
%     
%     %last traxel vertical connections
%     %Differ between input- and output-side depending on evenness of
%     %horizontal traxels
%     if NHTRE
%         for h=1:2:N(2)-1
%                 Uin(h*N(1):h*N(1),    1:2)=[3 (h+1)*N(1)];
%                 Uin((h+1)*N(1):(h+1)*N(1),    1:2)=[3 h*N(1)];
%         end
%     else
%         for h=1:2:N(2)-1
%                 Uou(h*N(1):h*N(1),    1:2)=[3 (h+1)*N(1)];
%                 Uou((h+1)*N(1):(h+1)*N(1),    1:2)=[3 h*N(1)];
%         end
%     end

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function to have all input connected to Uhig
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Uin]=AllInputConnected(N,Uhig)
    Uin(1:N(1)*N(2),1:1)=1;
    Uin(1:N(1)*N(2),2:2)=Uhig;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function to have all output connected to Ulow
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Uou]=AllOutputConnected(N,Ulow)
    Uou(1:N(1)*N(2),1:1)=1;
    Uou(1:N(1)*N(2),2:2)=Ulow;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function initialize some variables from the physical input
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Area,Zint_y,Zint_z,Rlat,Gamma_y,Gamma_z]=Initialize(Width,Height,Rho,Sigma_y,Sigma_z,epsilon,Ctrans_y,Ctrans_z,Freq)
    %First calculate some needed help variables
    Area=Width*Height;                   %Crossectional area of traxel in m^2
    
    %Resistances and capacities in y- and z-direction
    Rint_y=Sigma_y/Height;                   %Interface resistance in Ohm*m
    Rint_z=Sigma_z/Width;                   %Interface resistance in Ohm*m
    Cint_y=Ctrans_y*Height;                  %Interface Ctrans in F/m
    Cint_z=Ctrans_z*Width;                  %Interface Ctrans in F/m
    Zint_y=Rint_y/(1+1i*Freq*2*pi*Cint_y*Rint_y); %Interface impedance in Ohm/m
    Zint_z=Rint_z/(1+1i*Freq*2*pi*Cint_z*Rint_z); %Interface impedance in Ohm/m
    

    w=2*pi*Freq;    %frequency in rad
    
    Rlat=Rho/Area;                           %Lateral resistance in Ohm/m
    
    Gamma_y = Width./((1/Rho+1i*w*epsilon)).*(1+1i*w*Rho*epsilon+1i*w*Sigma_y*Ctrans_y-w.^2*Rho*epsilon*Sigma_y*Ctrans_y)./(Rho*Width+Sigma_y+1i*w*Rho*Sigma_y*Width*Ctrans_y+1i*w*Rho*Sigma_y*epsilon);
    Gamma_z = Height./((1/Rho+1i*w*epsilon)).*(1+1i*w*Rho*epsilon+1i*w*Sigma_z*Ctrans_z-w.^2*Rho*epsilon*Sigma_z*Ctrans_z)./(Rho*Height+Sigma_z+1i*w*Rho*Sigma_z*Height*Ctrans_z+1i*w*Rho*Sigma_z*epsilon);

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function to calculate the eigenvalues (eVac), eigenvectors (eVec) 
% coefficientss (Alpha) and impedance (Z) for a structure of Ntrax
% structure length (L), number of traxels (Ntrax), gamma value (G)
% and boundary conditions (Uin, Uou)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Lambda,eVec,Alpha,Z,Rlat,Gamma_y,Gamma_z]=CalcSolution(N,Length,Width,Height,Rho,Sigma_y,Sigma_z,epsilon,Ctrans_y,Ctrans_z,Freq,Uin,Uou)

%define some useful parameters and initiate matrices
Ntrax = N(1)*N(2);
Nlambda=2*Ntrax;
B(1:Nlambda,1:Nlambda)=0;
X(1:Nlambda)=0;
L2=0.5*Length;

%get fundamental constants and gammas
[Area,Zint_y,Zint_z,Rlat,Gamma_y,Gamma_z]=Initialize(Width,Height,Rho,Sigma_y,Sigma_z,epsilon,Ctrans_y,Ctrans_z,Freq);
% assignin('base','Gamma',Gamma);

%Fill matrix A with the coefficients from the coupled equations
A(1:Ntrax,1:Ntrax)=0;
for m=1:Ntrax
    for k=1:Ntrax
        if m==k 
            
            %conditions for assigning gammas to matrix
            if (mod(m-1,N(1))>0)
            A(m,k) = A(m,k)-Gamma_y/(Width^2);
            end
            
            if (mod(m-1,N(1))<N(1)-1)
            A(m,k) = A(m,k)-Gamma_y/(Width^2);
            end
            
            if ((m-N(1))>0)
            A(m,k) = A(m,k)-Gamma_z/(Height^2);
            end
            
            if ((m+N(1))<=Ntrax)
            A(m,k) = A(m,k)-Gamma_z/(Height^2);
            end
            
        else
            if m-k==1 && mod(k,N(1))~=0
                A(m,k)=A(m,k)+Gamma_y/(Width^2);
            elseif k-m==1 && mod(m,N(1))~=0
                A(m,k)=A(m,k)+Gamma_y/(Width^2);
            elseif abs(m-k)==N(1)
                    A(m,k)=A(m,k)+Gamma_z/(Height^2);
                    
            end
        end
    end
end
% assignin('base','A',A); 

%A is very sparse for large traxel numbers, so more efficient to use
%a sparse matrix explicitly
% non_zero_elements = Ntrax+2*(N(1)-1)*N(2)+2*(N(2)-1)*N(1);
A_sparse = sparse(A);
[eVec,eVal]=eigs(-A_sparse,Ntrax);
% assignin('base','A_sparse',A_sparse); 

%Calculate eigenvalues and eigenvectors. Note: it seems eig(A) calculates
%the eigenvalues of (A-lambda*I).
% [eVec,eVal]=eig(-A);
% eVal
% eVec

%Now sort the indices and eigenvectors
[d,ind]=sort(diag(eVal));
eVal=eVal(ind,ind);
assignin('base','eVal',eVal); 
eVec=eVec(:,ind);
assignin('base','eVec',eVec); 

% Generate the Lambda's from the eigenvalues.
for m=1:Ntrax
    Lambda(2*m-1)= sqrt(eVal(m,m));
    Lambda(2*m  )=-sqrt(eVal(m,m));
end
% Lambda

% Note: m(i,j) means m(row=i, column=j)
% Input conditions for x=0
for k=1:Ntrax           %Go over all layers
    switch Uin(k:k,1:1)
        case 1 %Voltage specified
            X(k)=Uin(k:k,2:2);
            B(k,1)= eVec(k,1);
            B(k,2)=-eVec(k,1)*L2;
            for m=3:Nlambda B(k,m)=eVec(k,ceil(m/2))*exp(-Lambda(m)*L2);
            end
        case 2 %Current is 0 condition
            X(k)=0;
            B(k,1)=0;
            B(k,2)=eVec(k,1);
            for m=3:Nlambda B(k,m)=eVec(k,ceil(m/2))*Lambda(m)*exp(-Lambda(m)*L2);
            end
        case 3 %Traxel inputs are connected
            X(k)=0;
            cnc=Uin(k:k,2:2);   %cnc=connecting channel
            if cnc>k            %apply voltage condition U_cnc-U_k=0. 
                B(k,1)= (eVec(cnc,1)-eVec(k,1));
                B(k,2)=-(eVec(cnc,1)-eVec(k,1))*L2;
                for m=3:Nlambda B(k,m)=(eVec(cnc,ceil(m/2))-eVec(k,ceil(m/2)))*exp(-Lambda(m)*L2);
                end 
            end
            if cnc<k      %Apply current condition I_k+I_cnc=0
                B(k,1)=0;
                B(k,2)=eVec(cnc,1)+eVec(k,1);
                for m=3:Nlambda B(k,m)=(eVec(cnc,ceil(m/2))+eVec(k,ceil(m/2)))*Lambda(m)*exp(-Lambda(m)*L2);
                end
            end
    end 
end;


% Note: m(i,j) means m(row=i, column=j)
% Output conditions for x=L
for k=1:Ntrax
    s=k+Ntrax;
    switch Uou(k:k,1:1)
        case 1 %Input voltage specified, current <> 0
            X(s)=Uou(k:k,2:2);
            B(s,1)=eVec(k,1);
            B(s,2)=eVec(k,1)*L2;
            for m=3:Nlambda B(s,m)=eVec(k,ceil(m/2))*exp(Lambda(m)*L2);
                % test to check to include lambda, incorrect but shows
                % problem real Z
%                 for m=1:Nlambda B(s,m)=eVec(k,ceil(m/2))*exp(-Lambda(m)*L2);
            end
        case 2 %No current, output not connected
            X(s)=0;
            B(s,1)=0;
            B(s,2)=eVec(k,1);
            for m=3:Nlambda B(s,m)=eVec(k,ceil(m/2))*Lambda(m)*exp(Lambda(m)*L2);
            end
        case 3 %Traxel inputs are connected
            X(s)=0;
            cnc=Uou(k:k,2:2);   %cnc=connecting channel
            if cnc>k            %apply voltage condition U_k-U_cnc=0. 
                B(s,1)=(eVec(cnc,1)-eVec(k,1));
                B(s,2)=(eVec(cnc,1)-eVec(k,1))*L2;
                for m=3:Nlambda B(s,m)=(eVec(cnc,ceil(m/2))-eVec(k,ceil(m/2)))*exp(Lambda(m)*L2);
                end 
            end
            if cnc<k      %Apply current condition I_k+I_cnc=0
                B(s,1)=0;
                B(s,2)=eVec(cnc,1)+eVec(k,1);
                for m=3:Nlambda B(s,m)=(eVec(cnc,ceil(m/2))+eVec(k,ceil(m/2)))*Lambda(m)*exp(Lambda(m)*L2);
                end
            end

    end
end;
Y=transpose(X);
Alpha=linsolve(B,Y);
% [Alpha,flag,~,~,~]=cg_method(B,Y,10e-3,1000);
assignin('base','Y',Y)
assignin('base','B',B)

assignin('base', 'Alpha', Alpha);
assignin('base', 'eVal', eVal);
assignin('base', 'eVec', eVec);

%Calculate impedance. First the current at the input and output
for k=1:Ntrax           %k runs over the number of traxels
    Iin(k)=Alpha(2)*eVec(k,1);
    Iou(k)=Alpha(2)*eVec(k,1);
    for q=3:Nlambda 
        Iin(k)=Iin(k)+Alpha(q)*eVec(k,ceil(q/2))*Lambda(q)*exp(-Lambda(q)*L2);
        Iou(k)=Iou(k)+Alpha(q)*eVec(k,ceil(q/2))*Lambda(q)*exp( Lambda(q)*L2);
    end
end
w = 2*pi*Freq;
Iin=-(Area/Rho +1i*Area*w*epsilon)*Iin;
Iou=-(Area/Rho +1i*Area*w*epsilon)*Iou;

Uhig=max(X);
Ulow=min(X);
Iinp=max(max(Iin),sum(Iin));
if real(Iinp)<0 Iinp=-1*Iinp; end;
Z=(Uhig-Ulow)/Iinp;

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function to calculate all voltages (U), currents (I) and powers (P) from
% the input structure length (L), number of traxels (N), gamma value (G)
% and boundary conditions (Uin, Uou)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [z,U,I,P,Pdens,Z]=CalcAllProfiles(N,Length,Width,Height,Rho,Sigma_y,Sigma_z,epsilon,Ctrans_y,Ctrans_z,Freq,Uin,Uou,Npoints)

%define some useful parameters 
Ntrax = N(1)*N(2);
L2=0.5*Length;
for m=1:Npoints 
    z(m)=(m-1)*Length/(Npoints-1); 
end
P(1:Ntrax,1:Npoints)=0;
deltaZ=Length/Npoints;
deltaA=deltaZ*Height;
Nlambda=2*Ntrax;

% [Area,Zint_y,Zint_z,Rlat,Gamma_y,Gamma_z]=Initialize(Width,Height,Rho,Sigma_y,Sigma_z);
Area = Width*Height;
[Lambda,eVec,Alpha,Z,Rlat,Gamma_y,Gamma_z]=CalcSolution(N,Length,Width,Height,Rho,Sigma_y,Sigma_z,epsilon,Ctrans_y,Ctrans_z,Freq,Uin,Uou);

% assignin('base','Z',Z)
% assignin('base','alphatest', Alpha)

% Calculate the voltages in the traxels as function of position
for k=1:Ntrax           %k runs over the number of traxels
    for m=1:Npoints     %m runs over the position in each traxel
        zp=z(m)-L2;
        U(k,m)=Alpha(1)*eVec(k,1)+Alpha(2)*eVec(k,1)*zp;
        for q=3:Nlambda U(k,m)=U(k,m)+Alpha(q)*eVec(k,ceil(q/2))*exp(Lambda(q)*zp);
        end
    end
end

assignin('base','U',U);       % write U voltage to workspace



% Calculate the currents in the traxels as function of position
for k=1:Ntrax           %k runs over the number of traxels
    for m=1:Npoints     %m runs over the position in each traxel
        zp=z(m)-L2;
        I(k,m)=Alpha(2)*eVec(k,1);
        for q=3:Nlambda I(k,m)=I(k,m)+Alpha(q)*eVec(k,ceil(q/2))*Lambda(q)*exp(Lambda(q)*zp);
        end
    end
end

%Calculate currents between traxels using impedance
dZc_y = 1/(Height*Length/Npoints)*( Rho*Width+Sigma_y) / (1);
dZc_z = 1/(Width*Length/Npoints)*( Rho*Height+Sigma_z) / (1);

I=-(Area/Rho)*I;
assignin('base', 'I', I/Area);

% Calculate power-density in traxels as function of position
% dZ = 1/(deltaA)*(((Rho*Width+Sigma+1i*2*pi*Freq*(Rho*Width*Sigma*Ctrans+Rho*epsilon*Sigma)))./((1+1i*2*pi*Freq*Rho*epsilon).*(1+1i*2*pi*Freq*Sigma*Ctrans)));

for k=1:Ntrax           %k runs over the number of traxels
    for m=1:Npoints     %m runs over the position in each traxel
        
        %Lateral
        if m>1
            P(k,m)= 1/2*abs(real( (U(k,m)-U(k,m-1))*conj(I(k,m)) ));
%              P(k,m)= 1/2*real(I(k,m)*conj(I(k,m))*dZb);
        elseif m<=1
            P(k,m)= 1/2*abs(real( (U(k,m+1)-U(k,m))*conj(I(k,m)) ));
%             P(k,m)= 1/2*real(I(k,m+1)*conj(I(k,m+1))*dZb);
        end
        
        %In y direction
        if k>1
%             P(k,m)=P(k,m)+1/2*0.5*abs(real( (U(k,m)-U(k-1,m)).*conj( U(k,m)-U(k-1,m) )./(conj(dZ)) ));          
            P(k,m)=P(k,m)+1/2*0.5*abs(real( (U(k,m)-U(k-1,m)).*conj( U(k,m)-U(k-1,m) )./(dZc_y) ));          
        end
        if k<Ntrax
%             P(k,m)=P(k,m)+1/2*0.5*abs(real((U(k,m)-U(k+1,m)).*conj(U(k,m)-U(k+1,m) )./(conj(dZ))));  
            P(k,m)=P(k,m)+1/2*0.5*abs(real((U(k,m)-U(k+1,m)).*conj(U(k,m)-U(k+1,m) )./(dZc_y)));  
        end
        
        %In z direction
        if (k-N(1))>1
            P(k,m)=P(k,m)+1/2*0.5*abs(real( (U(k,m)-U(k-N(1),m)).*conj( U(k,m)-U(k-N(1),m) )./(dZc_z) ));       
        end
        
        if (k+N(1))<Ntrax
            P(k,m)=P(k,m)+1/2*0.5*abs(real( (U(k,m)-U(k+N(1),m)).*conj( U(k,m)-U(k+N(1),m) )./(dZc_z) ));       
        end
    end
end

Pdens = P/(Width*Height*Length/Npoints);

Iinp=max(max(I(1:Ntrax,1:1)),sum(I(1:Ntrax,1:1)));
Ptot=0.5*real(Z*Iinp*conj(Iinp));

Ptot=sum(sum(P));
assignin('base', 'P', P);
assignin('base', 'Pdens', Pdens);

%turn I into I-density
I=I/Area;

%Z2=(Uhig-Ulow)^2/Ptot

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function to plot an impedance graph given frequencies and impedances
% Used internally in RunModel
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function f = Impedance_Graph(Z, Freq, fn)

%define font things
    FS=17;
    AFS=12;
    
%differ between specific figure or new one
    if fn == 0
        f = figure; hold on;
    else
        f = figure(fn); hold on;
    end

    
%plot magnitude of impedance
    subplot(2,1,1);
    plot(Freq,abs(Z),'LineWidth',2);
    ymin=10^floor(min(log10(abs(Z))));
    ymax=10^ceil(max(log10(abs(Z))));
    
    hold on
    Fmin = Freq(1);
    Fmax = Freq(size(Freq,2));
    axis([Fmin, Fmax, ymin, ymax]);
    title(strcat('Impedance vs Frequency'))
    xlabel('F [Hz]','fontsize',FS)
    ylabel('Mod(Z) [\Omega]','fontsize',FS);
    set(gca,'FontSize',AFS);
    set(gca, 'XScale', 'log');
    set(gca, 'YScale', 'log');
    grid;

%plot phase
    subplot(2,1,2);
    plot(Freq,angle(Z)/pi,'LineWidth',2);
    ymax=max(angle(abs(Z))/(0.5*pi))/2;
    ymin=min(angle(abs(Z))/(0.5*pi))/2;
    if ymax ~= ymin
        axis([Fmin Fmax ymin ymax]);
    end
    xlabel('F [Hz]','fontsize',FS)
    ylabel('Phase [Rad/pi]','fontsize',FS);
    set(gca,'FontSize',AFS);
    set(gca, 'XScale', 'log');
    grid;
    
end