%Function to create a 3D-model of voltage, current and power in one figure.
%The opacity of each part is determined by an alphamap.
%Only plots the real part of complex data
% U : (complex) double(N(1)*N(2), inf) 
% I : (complex) double(N(1)*N(2), inf) 
% P : (complex) double(N(1)*N(2), inf) 
% N : int Number of traxels in y- and z-direction [y,z]
% U_alpha_map : Alphamap for voltage. Can be a string to use a
% matlab-native map or a custom array. A good one here is 'vdown' which is
% (counterintuitively) a 'V'-shape making very large and small values
% visible.
% I_alpha_map : Alphamap for current. Can be a string to use a
% matlab-native map or a custom array. A good one here is 'rampup' which is
% just a linearly increasing map.
% P_alpha_map : Alphamap for power. Can be a string to use a
% matlab-native map or a custom array. A good one here is 'rampup'
% dims : double(3) = [Length, Width, Height] of individual traxels
% scale2cube : bool, causes scaling of visualization to a cube rather
% than displaying it realistically using the dims-argument
function f = Visualize_3D(U,I,P,N, freq, U_alpha_map, I_alpha_map, P_alpha_map, dims, scale2cube)
%     U_alpha_map  ='vdown';
%     I_alpha_map  ='rampup';
%     P_alpha_map  ='rampup';
%     N = [5,4];


%Define figure and set size
    f = figure;
    p = get(f, 'Position');
    set(f, 'Position' , [20, p(2), p(3)*3.3, p(4)]);
    
%make axes for each plot and plot with helper function below
    axU = subplot(1,3,1);
    title(axU, 'Voltage [U] = V @ ' + string(freq) + 'Hz');
    single_plot(U, N, axU, U_alpha_map, dims, scale2cube);
    colorbar;
    
    axI = subplot(1,3,2);
    title(axI, 'Current density [I] = A/m^2 @ ' + string(freq) + 'Hz');
    single_plot(I, N, axI, I_alpha_map, dims, scale2cube);
    colorbar;
    
    axP = subplot(1,3,3);
    title(axP, 'Power [P] = W @ ' + string(freq) + 'Hz');
    single_plot(P, N, axP, P_alpha_map, dims, scale2cube);
    colorbar;
    
    current_alpha_U = 0;
    p = get(f, 'Position');
    pU = get(axU, 'Position');
    pI = get(axI, 'Position');
    pP = get(axP, 'Position');  
    
%create controlling sliders and clickboxes for alphamap and cube-scaling
    sliders(1) = uicontrol('Parent',f, 'Style','slider','Position',[p(3)*pU(1)-50 p(4)*pU(2)+30 20, 250],...
              'value',current_alpha_U, 'min',-1, 'max',1);
    
    text(1) = uicontrol('Parent',f,'Style','text','Position',[p(3)*pU(1)-100 p(4)*pU(2)+140 45, 23],...
                    'String',string(current_alpha_U),'BackgroundColor',f.Color);
                
    c(1) = uicontrol('Style','checkbox','Min', false, 'Max', true,'String','Scale to cube','Position',[p(3)*pU(1)-100, p(4)*pU(2), 100, 20]);
    
    sliders(2) = uicontrol('Parent',f, 'Style','slider','Position',[p(3)*pI(1)-50 p(4)*pI(2)+30 20, 250],...
              'value',current_alpha_U, 'min',-1, 'max',1);
    
    text(2) = uicontrol('Parent',f,'Style','text','Position',[p(3)*pI(1)-100 p(4)*pI(2)+140 45, 23],...
                    'String',string(round(current_alpha_U)),'BackgroundColor',f.Color);
                
    c(2) = uicontrol('Style','checkbox','Min', false, 'Max', true,'String','Scale to cube','Position',[p(3)*pI(1)-100, p(4)*pI(2), 100, 20]);
                
    sliders(3) = uicontrol('Parent',f, 'Style','slider','Position',[p(3)*pP(1)-50 p(4)*pP(2)+30 20, 250],...
              'value',current_alpha_U, 'min',-1, 'max',1);
    
    text(3) = uicontrol('Parent',f,'Style','text','Position',[p(3)*pP(1)-100 p(4)*pP(2)+140 45, 23],...
                    'String',string(round(current_alpha_U)),'BackgroundColor',f.Color);
                
    c(3) = uicontrol('Style','checkbox','Min', false, 'Max', true,'String','Scale to cube','Position',[p(3)*pP(1)-100, p(4)*pP(2), 100, 20]);

%assign callbacks for alphamaps and cube-scaling
    sliders(1).Callback = @(es,ed) alpha_callback(es,ed,f,axU,U_alpha_map,text(1));
    c(1).Callback = @(es,ed) cube_callback(es,ed, U, N, axU, U_alpha_map, dims);
    
    sliders(2).Callback = @(es,ed) alpha_callback(es,ed,f,axI,I_alpha_map,text(2));
    c(2).Callback = @(es,ed) cube_callback(es,ed, I, N, axI, I_alpha_map, dims);
    
    sliders(3).Callback = @(es,ed) alpha_callback(es,ed,f,axP,P_alpha_map,text(3));
    c(3).Callback = @(es,ed) cube_callback(es,ed, P, N, axP, P_alpha_map, dims);
    
    
end


%% Helper Functions


%callback for alphamap-slider
function alpha_callback(es, ed, f, ax, alphatype, text)
    text.String = round(es.Value, 3);
    next_val = str2double(text.String);
%because of how the alphamap increase/decrease works internally it needs to be reset
%everytime and then changed with the correct function
    alphamap(ax, alphatype);
    if next_val > 0
        alphamap(ax, 'increase', next_val);
    else
        if next_val < 0
        alphamap(ax, 'decrease', -next_val);
        end
    end
end

%callback for scale-to-cube-button
function cube_callback(es, ed, data, N, ax, alpha_map, dims)
    cla(ax);
    %redraw the plot with the same properties but change the bool for
    %scaling to a cube
    single_plot(data, N, ax, alpha_map, dims, es.Value);
end

%Function to generate a 3D-plot for the given data and attach to axes "ax"
% data : Data to plot with dimension (N(1)*N(2), inf)
% N : Dimensions of data (x,y)
% ax : Axes-object to attach the plot to, allows usage of subplots in
% parent function
% alphatype : Controls the shape of the alphamap. This is necessary to
% avoid e.g. low voltage values to be invisible. Possible inputs are the
% same as the alphamaps that exist in native matlab. I recommend 'vdown' or 'default' for
% voltage, 'rampup' or 'vdown' for current and 'rampup' for power.
% dims : double(3) = [Length, Width, Height] of individual traxels
% scale2cube : bool, causes scaling of visualization to a cube rather
% than displaying it realistically using the dims-argument
function single_plot(data, N, ax, alphatype, dims, scale2cube)

%restructure data to make the plotting a little easier
data = restructure_data(data, N);
d = size(data);

%depending on the scale2cube 
if ~scale2cube
    Length = dims(1) / d(1);
    Width  = dims(2);
    Height = dims(3);
else
    Length = 1 / d(1);
    Width  = 1;
    Height = 1;
end

%Main meshgrid for sampling data
[x,y,z] = meshgrid((0:d(2)-1),(0:d(1)-1),(0:d(3)-1));

%make meshgrids for slices at bottom and end of x- and y-axes
[xz,yz] = meshgrid(0:d(2), 0:d(1));
zz = xz .* 0;

[xy,zy] = meshgrid(0:d(2), 0:d(3));
yy = xy .* 0 + d(1) - 1;

[yx,zx] = meshgrid(0:d(1), 0:d(3));
xx = yx .* 0 + d(2) - 1;

%initiate 3D plot
view(3);
grid on;
hold on;

%scaling matrix
scale = eye(4);
scale(1,1) = Width;
scale(2,2) = Length;
scale(3,3) = Height;

%transform object to apply to the slices
T = hgtransform('Parent', ax);
Ts = scale;
set(T,'Matrix',Ts)

%bottom slice
zslice = slice(ax, x,y,z,data,xz,yz,zz, 'nearest');
zslice.EdgeColor = 'none';
zslice.FaceAlpha = 0.5;
set(zslice, 'Parent', T);

%bottom slice but translated above the bottom traxel
%Note that this makes this slice appear twice but otherwise one of the
%sides of the cuboid has no surface
zslicetop = slice(ax, x,y,z,data,xz,yz,zz, 'nearest');
zslicetop.EdgeColor = 'none';
zslicetop.FaceAlpha = 0.3;
Tztop = hgtransform('Parent', ax);
set(zslicetop, 'Parent', Tztop);
ttop = Ts*makehgtform('translate',[0,0,1]);
set(Tztop,'Matrix',ttop);

%remaining x-y-plane slices
for i=1:d(3)-1
    zz = xz .* 0 + i;
    zslice(i+1) = slice(ax, x,y,z,data,xz,yz,zz, 'nearest');
    zslice(i+1).EdgeColor = 'none';
    zslice(i+1).FaceAlpha = 0.3;
    set(zslice(i+1), 'Parent', Tztop);
end

%y-slices
yslice = slice(ax, x,y,z,data,xy,yy,zy, 'nearest');
yslice.EdgeColor = 'none';
yslice.FaceAlpha = 0.5;
Tytop = hgtransform('Parent', ax);
set(yslice, 'Parent', Tytop);
tytop = Ts*makehgtform('translate',[0,1,0]);
set(Tytop,'Matrix',tytop);

yy = xy .* 0;
yslice2 = slice(ax, x,y,z,data,xy,yy,zy, 'nearest');
yslice2.EdgeColor = 'none';
yslice2.FaceAlpha = 0.3;
set(yslice2, 'Parent', T);

%front slice
xslice = slice(ax, x,y,z,data,xx,yx,zx, 'nearest');
xslice.EdgeColor = 'none';
xslice.FaceAlpha = 0.5;
set(xslice, 'Parent', T);
Txtop = hgtransform('Parent', ax);
set(xslice, 'Parent', Txtop);
txtop = Ts*makehgtform('translate',[1,0,0]);
set(Txtop,'Matrix',txtop);

%remaining slices
for i=0:d(2)-1
    xx = yx .* 0 + i;
    xslice(i+1) = slice(ax, x,y,z,data,xx,yx,zx, 'nearest');
    xslice(i+1).EdgeColor = 'none';
    xslice(i+1).FaceAlpha = 0.3;
    set(xslice(i+1), 'Parent', T);
end

colormap(ax, jet);

%lengthen axes in each direction to avoid 3D-confusion with outside/inside angles of
%cube. If you don't know what I mean try commenting it out, run it and
%rotate the plot a little, you'll understand quickly.
if scale2cube
    axis(ax, [-1, N(1)+1, -.1, 1.1, 0, N(2)]);
else
    maxl = max(max(Length*d(1),Width*N(1)),Height*N(2));
    axis(ax, [Width*N(1) / 2 - maxl / 2, Width*N(1) / 2 + maxl / 2, -maxl/10, maxl + maxl/10, Height*N(2)/2 - maxl/2, Height*N(2)/2 + maxl/2]);
end

%set alphamap
alpha(ax, 'color');
alphamap(ax, alphatype);
end

%Reorder data from (N(1)*N(2), size(U,2)) to (size(U,2), N(1), N(2))
%for 3D visualization.
function data = restructure_data(U,N)
    data = zeros([size(U,2),N(1),N(2)]);
    for i=1:N(2) 
        data(1:size(U,2),1:N(1),i) = real(U(1+(i-1)*(N(1)):i*N(1),:))';
    end
end